#ifndef __VIDEOFILE_LOAD_SW__
#define __VIDEOFILE_LOAD_SW__

////////////////////////////////////////////////////////////////////////////////////
//
// videofileload.h  
//
//  header file required for loading in video files created by the PITZ video system
//
////////////////////////////////////////////////////////////////////////////////////
//  created and modified by Stefan Weisse / DESY Zeuthen, November 2005
////////////////////////////////////////////////////////////////////////////////////

#include <videosystem3/platform.h>
#include <videosystem3/v2core.h>

#ifdef __cplusplus
extern "C" {
#endif 

// function prototypes

VIDEOSYSTEM3_API INT32  load_imm_file       ( const char *filename, struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  load_bkg_file       ( const char *filename, struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  load_imc_bkc_file   ( const char *filename, struct VIDINFHDR *vinf, INT32 type);
VIDEOSYSTEM3_API INT32  load_imc_file       ( const char *filename, struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  load_bkc_file       ( const char *filename, struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  load_file           ( INT32 type, const char *filename, struct VIDINFHDR *vinf );

VIDEOSYSTEM3_API const char * load_file__return_code_to_string(INT32 aiReturnCode, char* asOutString, size_t aiStringLengthBytes); // introduced 2023-02-15

// arbitrary, non-standard bkg file with multiple images
VIDEOSYSTEM3_API INT32  load_bkg_file_arb   ( const char *filename, struct VIDINFHDR *vinf ); 


// save_file, save_imm_file, save_bkg_file, save_bkc_file, save_imc_file, save_bmp_files
// - file(s) will be overwritten if already existing!
// - Return Values
//    0 successful
//
//   -1 multiple video frames can not be saved as bkg file (save_bkg_file)
//   -2 file type unknown
//   -3  error in vinf structure (either bad type or bad numframes (i.e. 0) )
    
//   -4  could not open file
//   -5  file write error !
//   -6  out of memory
//   -7  zlib compression error (save_imc_file, save_bkc_file)
//   -8  passed filename (and maybe path) is too big (bigger than MAXLEN-5) (save_file() )
//   -9  parameter error(check lzeroes[0..3], fcol[0..2], base_filename <>"") (save_bmp_files() )
//  -10  limitation: function works up to now only with 8bpp data inside vinf header! 
//       user programmer has to downscale data to 8 bit if necessary by oneself (save_bmp_files() )
//
VIDEOSYSTEM3_API INT32  save_file       ( INT32 type, const char *filename, const struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  save_imm_file   ( const char *outfilename, const struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  save_bkg_file   ( const char *outfilename, const struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  save_bkc_file   ( const char *outfilename, const struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  save_imc_file   ( const char *outfilename, const struct VIDINFHDR *vinf );
VIDEOSYSTEM3_API INT32  save_bmp_files  ( const char *base_filename, const struct VIDINFHDR *vinf, int fcol, int lzeroes);

// able to load in 8 bits per pixel (color table: 256 colors) video images as well as RGB24 bmp files
// vinf will return in any case 8 bits grayscale data (RGB24 image data will be downscaled (luminosity value) )
// 
// bEightBitDirect = 0 -> color table will be used and RGB24 will be created out of it, which
// will then be downscaled (luminosity value) to 8 bits per pixel
// bEightBitDirect = 1 -> 8 bits per pixel data will be directly loaded into 8 bits grayscale
VIDEOSYSTEM3_API INT32 load_bmp_file( const char *filename, struct VIDINFHDR *vinf, int aibEightBitDirect );

// SW Mar 19, 2013 added extended versions of saving possibly huge files START

// apdOutPercentageFinished: 0.0 (0%) to 1.0 (100%)
INT32 save_imm_file_internal(   const char *filename, 
                                const struct VIDINFHDR *vinf,
                                int *apiInInterrupt,
                                double *apdOutPercentageFinished
                            );

// apdOutPercentageFinished: 0.0 (0%) to 1.0 (100%)
INT32 save_imc_file_internal(   const char *filename, 
                                const struct VIDINFHDR *vinf,
                                int *apiInInterrupt,
                                double *apdOutPercentageFinished
                            );

// apdOutPercentageFinished: 0.0 (0%) to 1.0 (100%)
INT32 save_bkc_file_internal(   const char *filename, 
                                const struct VIDINFHDR *vinf,
                                int *apiInInterrupt,
                                double *apdOutPercentageFinished
                            );

// SW Mar 19, 2013 added extended versions of saving possibly huge files END

// SW Jun 23, 2022 added extended versions of loading possibly huge files

INT32  _internal_load_imm_file(const char *filename, 
    struct VIDINFHDR *vinf, 
    int *apiInInterrupt, 
    double *apdOutPercentageFinished, 
    double adPercentageMaximum);


INT32  _internal_load_bkg_file(const char *asFilename,
    struct VIDINFHDR *apVinf,
    double *apdOutPercentageFinished,
    double adPercentageMaximum);

// SW Jun 23, 2022 added extended versions of loading possibly huge files END

INT32  _internal_load_imc_file(const char *filename,
    struct VIDINFHDR *vinf,
    int *apiInInterrupt,
    double *apdOutPercentageFinished,
    double adPercentageMaximum);

#ifdef __cplusplus
}
#endif 

#endif  // __VIDEOFILE_LOAD_SW__
